package com.malk.aiwei.controller;

/**
 * 错误抛出与拦截详见 CatchException
 */

import com.alibaba.fastjson.JSON;
import com.malk.aiwei.service.AWClint;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.teambition.TBConf;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.service.teambition.TBClient;
import com.malk.utils.UtilMap;
import com.malk.utils.UtilServlet;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping
public class TBxYDController {

    @Autowired
    private AWClint awClint;

    /**
     * 交付物审批回调
     */
    @PostMapping("approved")
    McR approve(HttpServletRequest request) {

        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("交付物审批回调, {}", data);
        McException.assertParamException_Null(data, "projectId, taskId, creatorId, approve");

        awClint.approved(data);
        return McR.success();
    }

    /**
     * 检查项check回调
     */
    @PostMapping("checked")
    McR check(HttpServletRequest request) {

        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("检查项check回调, {}", data);
        McException.assertParamException_Null(data, "taskId, creatorId, formInstId");

        awClint.checked(data);
        return McR.success();
    }

    /**
     * 选项卡渲染获取详情
     */
    @PostMapping("load-check")
    public Map details(String taskId) {

        log.info("选项卡渲染, {}", taskId);

        String result = UtilMap.getString(awClint.doCheck(taskId, true), "result");
        String strJson = "{\"version\":\"1.0.0\",\"template\":{\"status\":\"#variables.status#\",\"toolbar\":[{\"label\":\"成功\",\"type\":\"text\",\"visible\":\"#variables.isDetailVisible#\",\"payload\":\"#variables.detailURL#\"}]},\"data\":[{\"itemId\":\"61b80257e1fe2200410c8ecd\",\"avatarUrl\":\"\",\"title\":\"xxxxx\",\"description\":\"\"}],\"variables\":{\"userId\":\"5bdc085d60bcef000146634f\",\"taskId\":\"\",\"status\":\"\",\"primaryLabel\":\"详情\",\"itemURL\":\"\",\"detailURL\":\"\",\"createURL\":\"\",\"createPlanTimeLabel\":\"\",\"createPlanTimeURL\":\"\",\"editURL\":\"\",\"removeURL\":\"\",\"isItemVisible\":true,\"isDetailVisible\":true,\"isCreatePlanTimeVisible\":true,\"isPrimaryVisible\":true,\"isEditVisible\":true,\"isRemoveVisible\":true}}";
        strJson = strJson.replace("xxxxx", result);
        return (Map) JSON.parse(strJson);
    }

    /**
     * 检查项发起
     */
    @PostMapping("do-check")
    McR doCheck(String taskId) {

        log.info("检查项发起, {}", taskId);
        return McR.success(awClint.doCheck(taskId, false));
    }

    /**
     * 通过模板创建项目
     */
    @PostMapping("project/create")
    McR createProject(HttpServletRequest request) {

        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("通过模板创建项目, {}", data);
        McException.assertParamException_Null(data, "projectCode, templateId");

        awClint.createProject(UtilMap.getString(data, "projectCode"), UtilMap.getString(data, "templateId"));
        return McR.success();
    }

    /**
     * 分配项目角色
     */
    @PostMapping("project/role")
    McR updateRoleProject(HttpServletRequest request) {

        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("分配项目角色, {}", data);
        McException.assertParamException_Null(data, "projectId", "trNode");

        awClint.updateProjectRole(UtilMap.getString(data, "projectId"), (List<String>) JSON.parse(UtilMap.getString(data, "trNode")));
        return McR.success();
    }

    /**
     * 删除前后置依赖
     */
    @PostMapping("project/remove/dependencies")
    McR removeDependencies(HttpServletRequest request) {

        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("删除前后置依赖, {}", data);
        McException.assertParamException_Null(data, "projectId", "trNode");

        awClint.removeDependencies(UtilMap.getString(data, "projectId"), (List<String>) JSON.parse(UtilMap.getString(data, "trNode")));
        return McR.success();
    }

    /**
     * 手动触发, 同步预检项
     */
    @PostMapping("check/sync")
    McR syncCheck(int type) {
        awClint.syncCheckList(type);
        return McR.success();
    }

    /**
     * 手动触发, 同步项目主数据
     */
    @PostMapping("project/sync")
    McR syncProject() {
        awClint.syncProject(null);
        return McR.success();
    }

    //////// test ////////

    @Autowired
    private TBClient tbClient;

    @Autowired
    private TBConf tbConf;

    @Autowired
    private DDConf ddConf;


    @GetMapping("t/task")
    McR task(String taskId) {

        log.info("taskId, {}", taskId);
        List<Map> tList = tbClient.queryTaskDetail(taskId, "", "");
        return McR.success(tList);
    }

    @GetMapping("t/template")
    McR template(String name) {

        List<Map> tList = tbClient.templateSearch(null, name);
        return McR.success(tList);
    }

    @Autowired
    private YDClient ydClient;

    @Autowired
    private YDService ydService;

    @PostMapping("test")
    McR test(HttpServletRequest request) {

        log.info("test, {}", UtilServlet.getParamMap(request));
        log.info("header, {}", UtilServlet.getHeaders(request));
        List<Map> rsp = tbClient.idMapQuery(tbConf.getOperatorId(), "dingTalk-user", ddConf.getCorpId());
        return McR.success(rsp);
    }

    // 获取项目自定义字段ID, 格式 { 名称: id } todo 提取
    Map<String, String> getProjectCFID(String projectId, List<String> names) {
        List<Map> customField = tbClient.queryProjectCustomField(projectId, null);
        List tList = customField.stream().filter(item -> names.contains(item.get("name"))).collect(Collectors.toList());
        return (Map<String, String>) tList.stream().reduce(UtilMap.empty(), (acc, cur) -> {
            ((Map) acc).put(((Map) cur).get("name"), ((Map) cur).get("id"));
            return acc;
        });
    }

    @GetMapping("tmp")
    McR tmp() {

        String projectId = "659376ba950780b816c33d3b";

//        tbClient.queryGanttBaseline(projectId, null);
//        String cfid = getProjectCFID(projectId, Arrays.asList(AWServer.TASK_CODE)).get(AWServer.TASK_CODE);
//        String tql = "cf:" + cfid + " = " + "TR1-01";
//        tbClient.queryProjectTaskList(projectId, UtilMap.map("q", tql), null);
//        tbClient.queryProjectRoles(projectId)
//        tbClient.queryProjectMember(projectId, null);

        awClint.test();
        return McR.success();
    }
}
