package com.malk.aiwei.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.aiwei.server.AWServer;
import com.malk.aiwei.service.AWClint;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.teambition.TBConf;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.service.teambition.TBClient;
import com.malk.utils.UtilEnv;
import com.malk.utils.UtilMap;
import com.malk.utils.UtilString;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class AWImplClient implements AWClint {

    @Autowired
    private YDClient ydClient;

    @Autowired
    private YDService ydService;

    @Autowired
    private YDConf ydConf;

    // 项目主数据表
    String _matchFormUuid(String code) {
        Map<String, String> formUuid = UtilMap.empty();
        if (UtilEnv.getActiveProfile().equals(UtilEnv.ENV_PROD)) {
            formUuid.put("REVIEW", "FORM-812FD46AF391449A8F206EDB3221B38840UQ"); // 交付物审批记录
            formUuid.put("REVIEW_PROCESS", "TPROC--RJC66SC1NEFHXJ0H770K0CF4WN1K21HQ706RL5"); // 交付物审批记录
            formUuid.put("PROJECT", "FORM-141E21DF183846028E21727CE43CD1C75CLZ"); // 项目主数据
            formUuid.put("APPROVE", "FORM-A25299893F614A6EAA672514D3A76BB0QDBF"); // 交付物审批矩阵
            formUuid.put("CHECK", "FORM-1A5D4D7FBF88409B956EBE51F9342A6BKOLP"); // 预检项
            formUuid.put("RECORD", "FORM-6E2C0D1197264B8AA23EB3FECAE7344B00BN"); // 预检项记录
            formUuid.put("ROLE", "FORM-3C7396A12ADB48A8833EBD90089C93833R21"); // 项目角色
            formUuid.put("DOMAIN", "https://yida.awinic.com/"); // 宜搭域名
        } else {
            formUuid.put("REVIEW", "FORM-FBC1A390B4C348089020C763938A6F54RUNY");
            formUuid.put("REVIEW_PROCESS", "TPROC--YU966T91PIDH4XDR82OJC8GVOP7Z19PODTXQL6");
            formUuid.put("PROJECT", "FORM-84EF78C7DBA047E58A8C8511106F91D5WNVI");
            formUuid.put("APPROVE", "FORM-AB7263D2A12F4E01A871656F0D995BC96PR1");
            formUuid.put("CHECK", "FORM-E6CB042D7929448888AE5E2B27631E57IVPM");
            formUuid.put("RECORD", "FORM-7B63BB056145452F8BC0A2C52492DE00QVBH");
            formUuid.put("ROLE", "FORM-5BE21392886E46DF955D1EBC100ADA429NON");
            formUuid.put("DOMAIN", "https://kabom7.aliwork.com/");
        }
        return formUuid.get(code);
    }

    /**
     * 交付物审批 [ppExt: 宜搭附件传递 downloadUrl 和 name 即可实现在线预览]
     */
    @Override
    public void doApprove(Map data) {

        log.info("交付物审批, {}", data);
        String projectId = UtilMap.getString(data, "projectId");
        String workFlowApprove = _getWorkFlowStatus(projectId, AWServer.WORKFLOW_APPROVE);
        if (!data.get("tfsId").equals(workFlowApprove)) {
            return;
        }
        String pCode = UtilMap.getString(data, "projectId");
        String creatorId = UtilMap.getString(data, "creatorId");
        String taskId = UtilMap.getString(data, "taskId");
        Map taskData = _getTaskFieldMap(UtilMap.getString(data, "taskId"), AWServer.TASK_CODE, AWServer.TASK_APPROVE_ATTACHMENT);
        String tCode = UtilMap.getString(taskData, AWServer.TASK_CODE);
        // 工作流tb虽然配置了流转逻辑, 冗余稳定性
        List<Map> formList = (List<Map>) ydClient.queryData(YDParam.builder()
                .formUuid(_matchFormUuid("REVIEW"))
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lqxuswzd, textField_lrncs2fu", tCode, pCode)))
                .instanceStatus("RUNNING")
                .build(), YDConf.FORM_QUERY.retrieve_search_process).getData();
        if (formList.size() > 0 && formList.stream().filter(item -> tCode.equals(((Map) item.get("data")).get("selectField_lqxuswzd"))).findAny().isPresent()) {
            return;
        }

        String result = "";
        List<Map> tList = null;
        Map rProject = null;
        if (StringUtils.isBlank(tCode)) {
            result = "无任务编码";
        } else {
            tList = ydService.queryDataList_FormData(_matchFormUuid("APPROVE"), UtilMap.map("selectField_lrncf4hk, radioField_lrnddfq6", tCode, "启用"));
            tList = tList.stream().filter(item -> tCode.equals(item.get("selectField_lrncf4hk"))).collect(Collectors.toList());
            if (tList.size() == 0) {
                result = "未配置交付物审批";
            } else {
                List<Map> pList = ydService.queryDataList_FormData(_matchFormUuid("PROJECT"), UtilMap.map("textField_lqxtykce", pCode));
                pList = pList.stream().filter(item -> pCode.equals(item.get("textField_lqxtykce"))).collect(Collectors.toList());
                if (pList.size() == 0) {
                    result = "未匹配到项目主数据";
                } else {
                    rProject = pList.get(0);
                }
            }
            /// 发起评审
            if (StringUtils.isBlank(result)) {

                List<Map> roles = (List<Map>) rProject.get("tableField_lqxtykcf");
                Map formData = UtilMap.map("selectField_lqxuswzd, textField_lrncs2fu, radioField_lrnddfq6", tCode, pCode, "启用");
                formData.put("attachmentField_lqxtebtq", _getDocs(taskData));
                formData.put("selectField_lqxuswze", UtilMap.getString(rProject, "textField_lrj7vnxb")); // 项目编号
                formData.put("textField_lqxuc9m4", UtilMap.getString(rProject, "textField_lqxtykcd")); // 项目名称
                // 匹配任务编码与项目角色
                List<Map<String, String>> compIds = Arrays.asList(  // 任务表角色, 交付物评审表: 角色, 审批人
                        UtilMap.map("tsRole, prRole, prEmp", "multiSelectField_lrokzlo7, multiSelectField_lrokzlo7, employeeField_lqxtebtw"),
                        UtilMap.map("tsRole, prRole, prEmp", "multiSelectField_lrokzlo8, multiSelectField_lrokzlo8, employeeField_lqxtebtx"),
                        UtilMap.map("tsRole, prRole, prEmp", "multiSelectField_lrokzlo9, multiSelectField_lrokzlo9, employeeField_lqxtebty"));
                Map rCondition = tList.get(0);
                for (Map compId : compIds) {
                    // ppExt: 宜搭多选若无值则不返回字段 [prd 角色多选, 会签逻辑]
                    if (!rCondition.containsKey(compId.get("tsRole"))) {
                        continue;
                    }
                    List<String> namesCondition = (List<String>) rCondition.get(compId.get("tsRole"));
                    List<String> namesApproveRole = new ArrayList<>();
                    List<String> namesApprover = new ArrayList<>();
                    for (String name : namesCondition) {
                        Optional optional = roles.stream().filter(item -> UtilString.equal(name, item.get("selectField_lqxu6bgf"))).findAny();
                        if (optional.isPresent()) {
                            Map detail = (Map) optional.get();
                            if (ObjectUtil.isNotNull(detail.get("selectField_lqxu6bgf"))) {
                                namesApproveRole.add(String.valueOf(detail.get("selectField_lqxu6bgf")));
                            }
                            if (ObjectUtil.isNotNull(detail.get("employeeField_lqxtykch_id"))) {
                                namesApprover.addAll((List) detail.get("employeeField_lqxtykch_id"));
                            }
                        }
                    }
                    formData.put(compId.get("prRole"), namesApproveRole);
                    formData.put(compId.get("prEmp"), namesApprover);
                }
                // 组装数据
                Map<String, String> extra = (Map) tbClient.idMapQuery(creatorId, "dingTalk-user", ddConf.getCorpId()).get(0).get("extra");
                UtilMap.putAll(formData, UtilMap.map("textField_lr3dlwsa, textField_lr3er4qb", taskId, creatorId));
                String instanceId = (String) ydClient.operateData(YDParam.builder()
                        .formUuid(_matchFormUuid("REVIEW"))
                        .processCode(_matchFormUuid("REVIEW_PROCESS"))
                        .formDataJson(JSON.toJSONString(formData))
                        .userId(extra.get("userId"))
                        .build(), YDConf.FORM_OPERATION.start);
                result = _matchFormUuid("DOMAIN") + ydConf.getAppType() + "/processDetail?procInsId=" + instanceId;
            }
        }
        log.info("交付物结果, {}", result);
        Map body = TBConf.assembleCustomFieldName(AWServer.TASK_APPROVE_LINK, result);
        tbClient.updateTaskCustomField(taskId, tbConf.getOperatorId(), body);
    }

    /**
     * 交付物审批回调
     */
    @Override
    public void approved(Map data) {

        log.info("交付物审批回调, {}", data);
        String result = String.valueOf(data.get("approve"));
        String workFlowId = _getWorkFlowStatus(String.valueOf(data.get("projectId")), result);
        tbClient.updateTaskFlowStatus(String.valueOf(data.get("taskId")), tbConf.getOperatorId(), workFlowId, String.valueOf(data.get("approve")));
    }

    @Autowired
    private TBClient tbClient;

    @Autowired
    private DDConf ddConf;

    // 获取任务详情 todo 提取到 tbservice
    private Map _getTaskFieldMap(String taskId, String... fieldNames) {

        Map rTask = tbClient.queryTaskDetail(taskId, null, null).get(0);
        List<Map> customfields = UtilMap.getList(rTask, "customfields");

        // 任务编码字段ID
        List<Map> customField = tbClient.queryProjectCustomField(String.valueOf(rTask.get("projectId")), null);

        // 组装任务数据
        Map result = UtilMap.map("task", rTask);
        for (String filed : fieldNames) {
            Optional optional = customField.stream().filter(item -> filed.equals(item.get("name"))).findAny();
            if (optional.isPresent()) {
                Map map = (Map) optional.get();
                result.put(filed, TBConf.getTaskFieldValue_First(customfields, String.valueOf(map.get("id"))));
                result.put(filed + "_id", map.get("id")); // ppExt: 返回字段ID
            }
        }
        return result;
    }

    // 匹配工作流名称, 获取ID
    @Deprecated
    String _getWorkFlowStatus(String projectId, String workFlowStatusName) {

        List<Map> customFlowStatus = tbClient.queryProjectCustomFlowStatus(projectId, UtilMap.map("q", workFlowStatusName));
        McException.assertAccessException(customFlowStatus.isEmpty(), "工作流名称未匹配");
        return String.valueOf(customFlowStatus.get(0).get("id"));
    }

    // 前置过滤 todo 提取方法
    List<String> _getWorkFlowStatusList(String projectId, String... workFlowStatusNames) {
        List<String> workFlowStatusList = new ArrayList<>();
        for (String name : workFlowStatusNames) {
            List<Map> customFlowStatus = tbClient.queryProjectCustomFlowStatus(projectId, UtilMap.map("q", name));
            workFlowStatusList.addAll(customFlowStatus.stream().map(item -> UtilMap.getString(item, "id")).collect(Collectors.toList()));
        }
        return workFlowStatusList;
    }

    // 后置过滤 todo 提取方法
    List<String> _getWorkFlowStatusList(String projectId, List<String> workFlowStatusNames) {
        //List<String> names = Arrays.stream(workFlowStatusNames).collect(Collectors.toList());
        List<Map> customFlowStatus = tbClient.queryProjectCustomFlowStatus(projectId, null);
        List<Map> workFlowStatusList = customFlowStatus.stream().filter(item -> workFlowStatusNames.contains(item.get("name"))).collect(Collectors.toList());
        return workFlowStatusList.stream().map(item -> UtilMap.getString(item, "id")).collect(Collectors.toList());
    }

    /**
     * 获取知识库附件传递到宜搭
     * - ppExt -
     * 1. 宜搭附件传递 downloadUrl 和 name 即可实现在线预览
     * 2. 知识库绑定没有文件后缀, 宜搭识别目前仅能点击下载跳转预览, 添加统一docx后缀, 文档会自行区分
     */
    List<Map> _getDocs(Map taskData) {
        Map rTask = UtilMap.getMap(taskData, "task");
        List<Map> customfields = UtilMap.getList(rTask, "customfields");

        String deliverable = String.valueOf(taskData.get(AWServer.TASK_APPROVE_ATTACHMENT + "_id")); // 交付物任务字段ID
        List<Map> attachments = TBConf.getTaskFieldValue(customfields, deliverable).stream().map(item -> {
            Map link = (Map) JSON.parse(String.valueOf(item.get("metaString")));
            return UtilMap.map("downloadUrl, name", link.get("url"), link.get("title") + ".docx");
        }).collect(Collectors.toList());
        return attachments;
    }

    /**
     * 检查项check
     *
     * @param isTask 服务于任务自定义详情卡片, 否则为宜搭发起: 抛出错误信息
     */
    @Override
    public Map doCheck(String taskId, boolean isTask) {

        log.info("检查项check, {}", taskId);
        Map taskData = _getTaskFieldMap(taskId, AWServer.TASK_CODE, AWServer.TASK_CHECK_LINK, AWServer.TASK_ROLE, AWServer.TASK_STAGE, AWServer.TASK_CHECK_STATUS);
        String checkLink = UtilMap.getString(taskData, AWServer.TASK_CHECK_LINK);
        /// prd 预检项持续维护需要重新匹配, 任务号和主数据判定为非法, 无需重新刷新
        if (isTask && (StringUtils.isNotBlank(checkLink) && !"未配置预检项".equals(checkLink))) {
            return UtilMap.map("result", "链接已加载");
        }
        // prd 非未完成任务若是未配置预检项状态, 忽略更新
        Map rTask = UtilMap.getMap(taskData, "task");
        if (isTask && !_getWorkFlowStatusList(UtilMap.getString(rTask, "projectId"), AWServer.WORKFLOW_INITIAL).contains(rTask.get("tfsId"))) {
            return UtilMap.map("result", "任务已执行");
        }
        String creatorId = UtilMap.getString(rTask, "creatorId");
        String tCode = UtilMap.getString(taskData, AWServer.TASK_CODE);
        String pCode = UtilMap.getString(rTask, "projectId");

        String result = "";
        List<Map> tList = null;
        Map rProject = null;
        if (StringUtils.isBlank(tCode)) {
            result = "无任务编码";
        } else {
            tList = ydService.queryFormData_all(YDParam.builder()
                    .formUuid(_matchFormUuid("CHECK"))
                    .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lrncf4hk, radioField_lrnddfq6", tCode, "启用")))
                    .build());
            tList = tList.stream().filter(item -> tCode.equals(item.get("selectField_lrncf4hk"))).collect(Collectors.toList());
            if (tList.size() == 0) {
                result = "未配置预检项";
            } else {
                List<Map> pList = ydService.queryDataList_FormData(_matchFormUuid("PROJECT"), UtilMap.map("textField_lqxtykce", pCode));
                pList = pList.stream().filter(item -> pCode.equals(item.get("textField_lqxtykce"))).collect(Collectors.toList());
                if (pList.size() == 0) {
                    result = "未匹配到项目主数据";
                } else {
                    rProject = pList.get(0);
                    result = _matchFormUuid("DOMAIN") + ydConf.getAppType() + "/workbench/" + _matchFormUuid("RECORD") + "?taskId=" + taskId;
                }
            }
        }
        log.info("检查项结果, {}", result);
        if (isTask) {
            if (!checkLink.equals(result)) {
                Map body = TBConf.assembleCustomFieldName(AWServer.TASK_CHECK_LINK, result);
                tbClient.updateTaskCustomField(taskId, tbConf.getOperatorId(), body);
            }
            // prd 未配置预检项更新为已检查, 避免完成任务完成触发必填校验 [ppExt 避免重复写入, 重复写入TB会重复记录日志
            if (!result.startsWith("http") && !taskData.get(AWServer.TASK_CHECK_STATUS).equals("已检查")) {
                Map body = TBConf.assembleCustomFieldName(AWServer.TASK_CHECK_STATUS, "已检查");
                tbClient.updateTaskCustomField(taskId, tbConf.getOperatorId(), body);
            }
            return UtilMap.map("result", result);
        }
        McException.assertAccessException(ObjectUtil.isNull(rProject), result);

        Map formData = UtilMap.map("selectField_lqxuswzd, textField_lrndwu00, textField_lr3dlwsa, textField_lr3er4qb", tCode, pCode, taskId, creatorId);
        // 项目信息
        formData.putAll(UtilMap.map("selectField_lqxuswze, textField_lrndwu07, textField_lqxuc9m4", UtilMap.getString(rProject, "textField_lrj7vnxb"), UtilMap.getString(rProject, "textareaField_lrj7vnxl"), UtilMap.getString(rProject, "textField_lqxtykcd")));
        // 任务信息 todo  userInfo 取值
        Map<String, String> extra = (Map) tbClient.idMapQuery(creatorId, "dingTalk-user", ddConf.getCorpId()).get(0).get("extra");
        formData.putAll(UtilMap.map("textField_lrndwu09, textField_lrndwu0a, textField_lrndwu0b, employeeField_lrndwu0e", rTask.get("content"), taskData.get(AWServer.TASK_STAGE), taskData.get(AWServer.TASK_ROLE), Arrays.asList(UtilMap.map("value, name", extra.get("userId"), extra.get("userId")))));
        // 返回检查项
        formData.put("tableField_lqxxgj4s", tList.stream().map(item -> {
            item.put("associationFormField_lrrnem5r", YDConf.associationForm(ydConf.getAppType(), _matchFormUuid("CHECK"), UtilMap.getString(item, "instanceId"), UtilMap.getString(item, "textField_lrnd3h0r"), "", false));
            return item;
        }).collect(Collectors.toList()));
        return formData;
    }

    /**
     * 检查项回调
     */
    @Override
    public void checked(Map data) {

        log.info("检查项回调, {}", data);
        Map body = TBConf.assembleCustomFieldName(AWServer.TASK_CHECK_STATUS, "已检查");
        tbClient.updateTaskCustomField(String.valueOf(data.get("taskId")), tbConf.getOperatorId(), body);
        String result = _matchFormUuid("DOMAIN") + ydConf.getAppType() + "/formDetail?formInstId=" + data.get("formInstId");
        Map body2 = TBConf.assembleCustomFieldName(AWServer.TASK_CHECK_LINK, result);
        tbClient.updateTaskCustomField(String.valueOf(data.get("taskId")), tbConf.getOperatorId(), body2);
    }

    /**
     * 同步项目主数据 [兼容新增]
     */
    @Synchronized
    @Override
    public Map syncProject(String projectCode) {

        // 新增场景下, 提供实例ID回写TB项目编号
        Map formData = null;

        // fixme: test生产环境, 人员组件不能同步, 更新后人员会为空. todo 测试环境建立模拟项目主数据单据
        // 查询项目主数据 [艾为应用]
        YDParam ydParam = YDParam.builder()
                .appType("APP_QBWQITQBSPJNYTUTNPDK")
                .systemToken("8F966HB12J27MQJM6V4IQDYHYTPA2G4GTZGCLN1")
                .formUuid("FORM-Q4A664A1V158R81L662N9507PLND288T7O6ELN")
                .build();
        if (StringUtils.isNotBlank(projectCode)) {
            ydParam.setSearchFieldJson(JSON.toJSONString(UtilMap.map("textField_le6o88w0", projectCode)));
        }
        // prd 客户系统主数据, 是全量定时从mdm更新, 因此不能增量
        List<Map> mapBaseList = ydService.queryFormData_all(ydParam);
        if (StringUtils.isNotBlank(projectCode)) {
            mapBaseList = mapBaseList.stream().filter(item -> projectCode.equals(item.get("textField_le6o88w0"))).collect(Collectors.toList());
        }
        log.info("项目主数据, {}", mapBaseList.size());
        McException.assertAccessException(mapBaseList.isEmpty(), "未匹配项目主数据");

        // 查询项目角色
        YDParam ydParam1 = YDParam.builder()
                .formUuid(_matchFormUuid("ROLE"))
                .build();
        List<Map> mapRool = (List<Map>) ydClient.queryData(ydParam1, YDConf.FORM_QUERY.retrieve_list).getData();

        // 匹配数据组件ID
        for (Map baseFormData : mapBaseList) {
            // 项目主数据
            formData = new HashMap();
            formData.put("textField_lqxtykcd", baseFormData.get("textField_lo2c1f0l"));      //项目名称
            formData.put("textField_lrj7vnxb", baseFormData.get("textField_le6o88w0"));      //项目号
            formData.put("textField_lrj7vnxc", baseFormData.get("textField_le6o88w1"));      //项目简称
            formData.put("textareaField_lrj7vnxl", baseFormData.get("textField_le6o88w2"));  //项目描述
            formData.put("textField_lrj7vnxf", baseFormData.get("formInstanceId"));          //项目实例ID
            // 项目角色详情
            List<Map> details = new ArrayList<>();
            mapRool.forEach(root -> {
                Map rootFormData = (Map) root.get("formData");
                // 创建详情的角色名称和成员组件Map
                Object rootItems = baseFormData.get(rootFormData.get("textField_lr7bgi76"));
                Object projectRoot = rootFormData.get("textField_lqxu439g");
                if (rootItems != null && projectRoot != null) {
                    details.add(UtilMap.map("employeeField_lqxtykch, selectField_lqxu6bgf", rootItems, projectRoot));
                }
            });
            formData.put("tableField_lqxtykcf", details);

            // 通过项目号获取项目是否已存在
            List<Map> projectMaps = ydService.queryDataList_FormData(_matchFormUuid("PROJECT"), UtilMap.map("textField_lrj7vnxb", baseFormData.get("textField_le6o88w0")));
            projectMaps = projectMaps.stream().filter(sub -> baseFormData.get("textField_le6o88w0").equals(sub.get("textField_lrj7vnxb"))).collect(Collectors.toList());

            YDParam ydParam2 = YDParam.builder()
                    .formUuid(_matchFormUuid("PROJECT"))
                    .formDataJson(JSON.toJSONString(formData))
                    .updateFormDataJson(JSON.toJSONString(formData))
                    .useLatestVersion(true)
                    .build();
            if (projectMaps.size() > 0) {
                ydParam2.setFormInstanceId(String.valueOf(projectMaps.get(0).get("instanceId")));
                ydClient.operateData(ydParam2, YDConf.FORM_OPERATION.update);
                // 保留tb项目号, 避免冲重复通过模板参加项目
                formData.put("textField_lqxtykce", UtilMap.getString(projectMaps.get(0), "textField_lqxtykce"));
                formData.put("formInstanceId", projectMaps.get(0).get("instanceId"));
            } else {
                String formInstId = (String) ydClient.operateData(ydParam2, YDConf.FORM_OPERATION.create);
                formData.put("formInstanceId", formInstId);
            }
            log.info("同步项目主数据", projectCode);
        }
        // tb项目成员更新
        String projectId = UtilMap.getString(formData, "textField_lqxtykce");
        if (StringUtils.isBlank(projectId) && StringUtils.isBlank(projectCode)) {
            _syncProjectRole(formData, projectId);
        }
        return formData;
    }


    @Autowired
    private TBConf tbConf;

    /**
     * 通过模板创建项目 [ppExt: 过滤宜搭项目档案\TB项目, 避免重复创建, 若判定为更新下执行项目成员更新]
     */
    @Override
    @Synchronized
    public void createProject(String projectCode, String templateId) {

        log.info("通过模板创建项目, {}, {}", projectCode, templateId);

        // 虽然先创建TB, 可以一次性同步项目数据, 但避免执行异常, 因此成功创建TB项目再回写
        Map formData = syncProject(projectCode);
        log.info("项目主数据, {}", formData);

        String projectId = UtilMap.getString(formData, "textField_lqxtykce");
        if (StringUtils.isBlank(projectId)) {
            // 通过模板创建项目, 创建项目并更新项目TB项目ID
            Map result = tbClient.projectCreateWithTemplate(projectCode, templateId, tbConf.getOperatorId());
            projectId = UtilMap.getString(result, "id");
            ydClient.operateData(YDParam.builder()
                    .formInstanceId(UtilMap.getString(formData, "formInstanceId"))
                    .updateFormDataJson(JSON.toJSONString(UtilMap.map("textField_lqxtykce", projectId)))
                    .build(), YDConf.FORM_OPERATION.update);
        }
        _syncProjectRole(formData, projectId);
    }

    // 同步TB项目成员 todo 群成员
    void _syncProjectRole(Map formData, String projectId) {
        // prd TB项目成员, 项目经理设置为管理者
        List<String> pmUserId = new ArrayList<>();
        List<String> roleIds = new ArrayList<>();
        List<Map> details = (List<Map>) formData.get("tableField_lqxtykcf");
        for (Map row : details) {
            // 同步项目主数据, 会重新匹配项目成员, 因此无需 employeeField_xxx_id 取值
            List<String> userIds = (List<String>) UtilMap.getList(row, "employeeField_lqxtykch");
            if (userIds.isEmpty()) {
                continue;
            }
            if (AWServer.PROJECT_PM_NAME.equals(row.get("selectField_lqxu6bgf"))) {
                pmUserId.addAll(userIds);
            }
            roleIds.addAll(userIds);
        }
        // 钉钉人员ID转TB人员ID
        roleIds = _convertUserId(roleIds, false);
        tbClient.createProjectMember(projectId, roleIds, tbConf.getOperatorId());

        // prd 不做删除, 离职自动退出, 避免临时交接不可见的问题. 但需要移除项目经理权限
        String pmRoleId = _getProjectRoleId(projectId, AWServer.PROJECT_PM_ROLE);
        List<Map> roleList = tbClient.queryProjectMember(projectId, UtilMap.map("projectRoleId", pmRoleId));
        for (Map role : roleList) {
            List<String> tRoleIds = UtilMap.getList(role, "roleIds");
            tRoleIds.remove(pmRoleId);
            if (tRoleIds.isEmpty()) {
                tRoleIds.add(_getProjectRoleId(projectId, "项目成员"));  // 极少情况, 无需冗余
            }
            tbClient.updateProjectMember(Arrays.asList(UtilMap.getString(role, "userId")), tRoleIds, projectId);
        }
        // 指定项目经理为编辑权限
        if (!pmUserId.isEmpty()) {
            tbClient.updateProjectMember(_convertUserId(pmUserId, false), Arrays.asList(pmRoleId), projectId);
        }
        log.info("TB项目信息, {}, {}", projectId, roleIds.size());
    }

    // todo 提取方法, 参考 getProjectCFID 实现
    private String _getProjectRoleId(String projectId, String roleName) {
        List<Map> roles = tbClient.queryProjectRoles(projectId);
        Optional optional = roles.stream().filter(item -> roleName.equals(item.get("name"))).findAny();
        McException.assertAccessException(!optional.isPresent(), roleName + ": 项目角色不存在");
        return UtilMap.getString((Map) optional.get(), "id");
    }

    /**
     * 分配项目角色 prd 若是一人直接指定, 多人情况下忽略
     */
    @Synchronized
    @Override
    public void updateProjectRole(String projectId, List<String> trNode) {

        // 项目档案 [tb项目号查询, 避免模糊匹配]
        List<Map> pList = ydService.queryDataList_FormData(_matchFormUuid("PROJECT"), UtilMap.map("textField_lqxtykce", projectId));
        McException.assertAccessException(pList.isEmpty(), "未匹配到项目主数据");
        List<Map> rList = (List<Map>) pList.get(0).get("tableField_lqxtykcf");

        log.info("分配项目角色 projectId = {}, 项目数量 = {}, 角色列表 = {}", projectId, pList.size(), rList);
        // 任务编码字段ID
        List<Map> customField = tbClient.queryProjectCustomField(projectId, null);
        // prd 非未完成任务, 不执行人员匹配更新
        List<String> workFlowStatusList = _getWorkFlowStatusList(projectId, AWServer.WORKFLOW_INITIAL);

        //List<Map> taskList = tbClient.queryProjectTaskList(projectId, UtilMap.map("pageSize", 10), null);
        tbClient.queryProjectTaskList(projectId, null, taskList -> {
            for (Map task : taskList) {
                if (!workFlowStatusList.contains(task.get("tfsId"))) {
                    continue;
                }
                // 获取资源名称对应的项目角色
                Optional optional = customField.stream().filter(item -> AWServer.TASK_ROLE.equals(item.get("name"))).findAny();
                if (!optional.isPresent()) {
                    continue;
                }
                List<Map> customfields = (List<Map>) task.get("customfields");
                // prd 按照tr节点分配成员, 若是含有全部则忽略
                if (!trNode.contains("全部")) {
                    Optional optional2 = customField.stream().filter(item -> AWServer.TASK_STAGE.equals(item.get("name"))).findAny();
                    if (!optional2.isPresent()) {
                        continue;
                    }
                    String trName = TBConf.getTaskFieldValue_First(customfields, UtilMap.getString((Map) optional2.get(), "id"));
                    if (!trNode.contains(trName)) {
                        continue;
                    }
                    log.info("trName = {}", trName);
                }
                String roleName = TBConf.getTaskFieldValue_First(customfields, UtilMap.getString((Map) optional.get(), "id"));
                // 获取角色在项目主数据对应成员
                optional = rList.stream().filter(item -> roleName.equals(item.get("selectField_lqxu6bgf"))).findAny();
                if (!optional.isPresent()) {
                    continue;
                }
                log.info("分配项目角色 taskId = {}, trNode = {}, 资源名称 = {}, 项目角色 = {}", UtilMap.getString(task, "id"), roleName, optional.get());
                List<String> roleIds = (List<String>) UtilMap.getList((Map) optional.get(), "employeeField_lqxtykch_id");
                if (roleIds.size() == 1) {
                    String tbUserId = _convertUserId(roleIds.get(0), false);
                    if (StringUtils.isNotBlank(tbUserId)) {
                        tbClient.updateTaskExecutor(UtilMap.getString(task, "id"), tbConf.getOperatorId(), tbUserId, false, false);
                    }
                }
            }
            return true;
        });
    }

    /**
     * 项目迁移: 删除依赖项
     */
    @Synchronized
    @Override
    public void removeDependencies(String projectId, List<String> trNode) {

        // 任务编码字段ID
        List<Map> customField = tbClient.queryProjectCustomField(projectId, null);
        tbClient.queryProjectTaskList(projectId, null, taskList -> {
            for (Map task : taskList) {

                Optional optional = customField.stream().filter(item -> AWServer.TASK_STAGE.equals(item.get("name"))).findAny();
                if (!optional.isPresent()) {
                    continue;
                }
                List<Map> customfields = (List<Map>) task.get("customfields");
                String trName = TBConf.getTaskFieldValue_First(customfields, UtilMap.getString((Map) optional.get(), "id"));
                if (trNode.contains(trName)) {
                    String taskId = UtilMap.getString(task, "id");
                    List<Map> dependencies = tbClient.queryTaskDependency(taskId, null);
                    log.info("taskID = {}, trNode, {}, 依赖数量: {}", taskId, trName, dependencies.size());
                    for (Map dependency : dependencies) {
                        tbClient.removeTaskDependency(UtilMap.getString(dependency, "id"), tbConf.getOperatorId());
                    }
                }
            }
            return true;
        });
    }

    // TB与宜搭userId转换 todo 提取 tbService
    private String _convertUserId(String userId, boolean isTBID) {
        List<Map> tbMap = tbClient.idMapQuery(userId, isTBID);
        // 过滤未匹配人员信息 [ppExt: TB人员未匹配, 不执行修改也无报错]
        if (tbMap.isEmpty()) {
            log.warn("TB与宜搭userId转换为空, userId = {}, tbID = {}", userId, isTBID);
            return "";
        }
        if (!isTBID) {
            return UtilMap.getString(tbMap.get(0), "tbId");
        } else {
            Map extra = UtilMap.getMap(tbMap.get(0), "extra");
            return UtilMap.getString(extra, "userId");
        }
    }

    // TB与宜搭userId转换 todo 提取 tbService
    private List<String> _convertUserId(List<String> userIds, boolean isTBID) {
        return userIds.stream().distinct()
                .map(id -> _convertUserId(id, isTBID))
                .filter(id -> StringUtils.isNotBlank(id))
                .collect(Collectors.toList());
    }

    /**
     * 同步预检项 [全量同步, 忽略任务号为空记录]
     */
    @Synchronized
    @Override
    public void syncCheckList(YDParam srcParam, Map<String, ?> compIds, String taskCompId, String codeCompId, String checkType, String associationCompId) {

        String curFormUuid = "FORM-1A5D4D7FBF88409B956EBE51F9342A6BKOLP";
        // 数据来源表
        List<Map> srcList = ydService.queryFormData_all(srcParam);
        // 当前记录表
        List<Map> curList = ydService.queryFormData_all(YDParam.builder()
                .formUuid(curFormUuid)
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lrnd3h0s", checkType)))
                .build());

        for (Map item : srcList) {
            if (StringUtils.isBlank(UtilMap.getString(item, taskCompId))) {
                continue;
            }
            Map formData = UtilMap.map(associationCompId, YDConf.associationForm(srcParam.getAppType(), srcParam.getFormUuid(), UtilMap.getString(item, "formInstanceId"), UtilMap.getString(item, codeCompId), "", false));
            formData.put("radioField_lrnddfq6", "启用");
            formData.put("selectField_lrnd3h0s", checkType);
            for (String compId : compIds.keySet()) {
                formData.put(compId, item.get(compIds.get(compId)));
            }
            YDConf.FORM_OPERATION operate = YDConf.FORM_OPERATION.create;
            YDParam ydParam = YDParam.builder()
                    .formUuid(curFormUuid)
                    .formDataJson(JSON.toJSONString(formData))
                    .updateFormDataJson(JSON.toJSONString(formData))
                    .useLatestVersion(true)
                    .build();
            // 避免循坏内查询, 也避免了模糊匹配
            Optional optional = curList.stream().filter(cur -> cur.get("textField_lrnd3h0r").equals(item.get(codeCompId))).findAny();
            if (optional.isPresent()) {
                ydParam.setFormInstanceId(UtilMap.getString((Map) optional.get(), "formInstanceId"));
                operate = YDConf.FORM_OPERATION.update;
            }
            ydClient.operateData(ydParam, operate);
        }
    }

    /**
     * 同步预检项 [通用]
     */
    @Synchronized
    @Override
    public void syncCheckList(int type) {

        switch (type) {
            case 0:
                // 经验库
                YDParam ydParam1 = YDParam.builder()
                        .appType("APP_D4NPOH9J3KCG5VE1RVN6")
                        .systemToken("6J8668A1B59FA7SAFTAF89PTKKGP2HY3FP3OLV1")
                        .formUuid("FORM-D47CCA076DA54F969DF7442D81E4B5A8PC6Q")
                        .build();
                // 任务号, 描述, ID（标题）
                Map<String, ?> compIds1 = UtilMap.map("selectField_lrncf4hk, textField_lrrnqz7w, textField_lrnd3h0r", "selectField_ls9yctsd", "textareaField_lo5eakcv", "serialNumberField_lo5dmyfb");
                syncCheckList(ydParam1, compIds1, "selectField_ls9yctsd", "serialNumberField_lo5dmyfb", "经验库", "associationFormField_lrrn5csf");
                break;

            case 1:
                // 技术评审要素表
                YDParam ydParam2 = YDParam.builder()
                        .appType("APP_NOSSCZQ8FFZKHDQOIIDA")
                        .systemToken("UM6660D1BFTFLFAP7NPJDBJEEAB737FAR1SOLD4")
                        .formUuid("FORM-B40420AFCDB84AD69FE78D82D6D5CFD2C2D2")
                        .build();
                // 任务号, 描述, ID（标题）
                Map<String, ?> compIds2 = UtilMap.map("selectField_lrncf4hk, textField_lrrnqz7w, textField_lrnd3h0r", "selectField_bclg9y5", "textField_k1e08ji", "textField_96ikaoh");
                syncCheckList(ydParam2, compIds2, "selectField_bclg9y5", "textField_96ikaoh", "技术评审要素表", "associationFormField_lrrn5csg");
                break;

            case 2:
                // TR评审要素表
                YDParam ydParam3 = YDParam.builder()
                        .formUuid("FORM-5436DC75BC8347D498DFF2617925BA70O066")
                        .build();
                // 任务号, 描述, ID（标题）, 验收标准, 等级
                Map<String, ?> compIds3 = UtilMap.map("selectField_lrncf4hk, textField_lrrnqz7w, textField_lrnd3h0r, textField_cd4f65l, textField_267kho2", "selectField_4gwjfa5, textField_lryetn3g, textField_9x1yps6, textField_cd4f65l, textField_267kho2");
                syncCheckList(ydParam3, compIds3, "selectField_4gwjfa5", "textField_9x1yps6", "TR评审要素表", "associationFormField_lrrn5csa");
                break;
        }
    }

    @Override
    public void test() {

    }
}


